#!/usr/bin/env python
# Copyright 2016 Mellanox Technologies, Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import yaml
import os
import time
import sys
import argparse
import textwrap
import logging
from params import EnvParams
from params import OvercloudParams
from utils import run_command
from utils import load_file
from utils import check_bool

logging.basicConfig(format='%(asctime)s %(filename)s %(message)s',
                    level=logging.DEBUG, filename=EnvParams.LOG_FILE_PATH)

parser = argparse.ArgumentParser(
    prog=os.path.basename(sys.argv[0]),
    formatter_class=argparse.RawDescriptionHelpFormatter,
    description=textwrap.dedent('''\
             Available features to choose:
             -------------------------------
                - SRIOV
                - iSER
                - VXLAN
                - NEO

            Choose the required configurations in {0}

             '''.format(EnvParams.CONF_FILE_NAME)))

args = parser.parse_args()

# Load configurations
params = load_file(EnvParams.CONF_FILE_PATH)
sriov = params['Sriov']
neutron = params['Neutron']
vxlan = neutron['network_type'].lower() == 'vxlan'
iser = params['Iser']
neo = params['Neo']
bond = params['Bond']
interfaces_validation = params['InterfacesValidation']

# Check boolean values
sriov['sriov_enable'] = check_bool(sriov['sriov_enable'])
iser['iser_enable'] = check_bool(iser['iser_enable'])
neo['neo_enable'] = check_bool(neo['neo_enable'])
bond['bond_enable'] = check_bool(bond['bond_enable'])
interfaces_validation = check_bool(interfaces_validation)


def check_rules():

    # Check configurations validity
    if sriov['sriov_enable'] is True:
        if sriov['number_of_vfs'] is None:
            parser.error("enabling sriov requires number_of_vfs.")
        if sriov['number_of_vfs'] < 1 or sriov['number_of_vfs'] > 32:
            parser.error('num_vfs must be less or equal 32')
        if vxlan is True:
            parser.error("Cannot enable SRIOV and VXLAN at the same time.")

    if not sriov['sriov_enable']:
        if sriov['number_of_vfs'] is not None and sriov['number_of_vfs'] != 0:
            parser.error("number_of_vfs requires enabling sriov.")
        sriov['number_of_vfs'] = 0

    if neo['neo_enable'] is True:
        if neo['neo_ip'] is None:
            parser.error("Enabling NEO requires neo_ip")
        if neo['neo_username'] is None:
            parser.error("Enabling NEO requires neo_user")
        if neo['neo_password'] is None:
            parser.error("Enabling NEO requires neo_password")

    check_interfaces_validity(interfaces_validation)


def check_interfaces_validity(interfaces_validation):

    if interfaces_validation is True:

        # Check interfaces validity
        interfaces = load_file(EnvParams.INTERFACES_DRIVERS_YAML_FILE_PATH)
        interfaces_list = []
        new_names_interfaces_list = []

        for key in interfaces:
            for port in interfaces[key]['interfaces']:
                interfaces_list.append(port)
                if 'new_name' in interfaces[key]['interfaces'][port]:
                    new_name = interfaces[key]['interfaces'][port]['new_name']
                    if new_name.strip():
                        new_names_interfaces_list.append(
                            interfaces[key]['interfaces'][port]['new_name'])

        if sriov['sriov_interface'] not in interfaces_list \
                and sriov['sriov_interface'] not in new_names_interfaces_list:
            parser.error('Please choose a valid port from {0}'.format(
                         EnvParams.INTERFACES_DRIVERS_YAML_FILE_PATH))

        if iser['storage_network_port'] not in interfaces_list \
                and iser['storage_network_port'] not in \
                new_names_interfaces_list:
            parser.error('Please choose a valid port from {0}'.format(
                         EnvParams.INTERFACES_DRIVERS_YAML_FILE_PATH))

        if iser['iser_enable'] and iser['storage_network_port'] in \
                interfaces_list and not interfaces[key]['interfaces'][
                iser['storage_network_port']]['driver'].startswith('mlx'):
            parser.error(
                'Storage interface is not RDMA capable')

        if sriov['sriov_interface'] == iser['storage_network_port']:
            parser.error(
                'Cannot use same interface for sriov_interface and ' +
                'storage_network_port in config.yaml')


def generate_yaml():

    # grep package version
    version_out = run_command(
        "grep -w 'version:' {0}".format(EnvParams.METADATA_FILE_PATH))
    version_list = version_out.rsplit()
    version_index = version_list.index("version:") + 1
    version = version_list[version_index].rstrip()

    env_config = load_file(EnvParams.ENV_TEMPLATE_YAML_FILE_PATH)
    sriov_config = load_file(EnvParams.NEUTRON_SRIOV_TEMPLATE_YAML_FILE_PATH)

    # Add interfaces and flavors
    env_config['parameter_defaults']['ExtraConfig'][
        'mellanox-rhel-osp'].update(
        {
            'iser_ifc_name': iser['storage_network_port'],
            'physical_port': sriov['sriov_interface'],
            'sriov': sriov['sriov_enable'],
            'iser': iser['iser_enable'],
            'use_mlnx_neo': neo['neo_enable'],
            'vxlan_offloading': vxlan,
            'num_of_vfs': sriov['number_of_vfs'],
            'package_version': version,
            'neutron_sriov_network': neutron['physnet_vlan']
        })

    if sriov['sriov_enable'] is True:
        sriov_config['parameter_defaults'].update({
            'NeutronPhysicalDevMappings': '{0}:{1}'.format(
                                          neutron['physnet_vlan'],
                                          sriov['sriov_interface']),
            'NeutronSriovNumVFs': '{0}:{1}'.format(
                                  sriov['sriov_interface'],
                                  sriov['number_of_vfs']),
            'NovaPCIPassthrough': [{'devname': sriov['sriov_interface'],
                                    'physical_network':
                                    neutron['physnet_vlan']}]
        })

        # Generate environment configuration yaml
        with open(EnvParams.NEUTRON_SRIOV_YAML_FILE_PATH, "w") as f:
            yaml.dump(
                sriov_config,
                f,
                default_flow_style=False,
                width=4,
                indent=4)

        output_msg = "Generated {0} SR-IOV configurations file".format(
            EnvParams.NEUTRON_SRIOV_YAML_FILE_PATH) \
            + " to be used in overcloud deployment under {0}".format(
             EnvParams.ENV_CONF_DIR_PATH)
        logging.info(output_msg)
        print output_msg

    # Add NEO configurations
    if neo['neo_enable'] is True:
        env_config['parameter_defaults']['ExtraConfig'][
            'mellanox-rhel-osp'].update(
            {
                'mlnx_neo_ip': neo['neo_ip'],
                'mlnx_neo_password': neo['neo_password'],
                'mlnx_neo_user': neo['neo_username'],
                'restart_service': OvercloudParams.RESTART_SERVICE_PATH
            })

    # Add VXLAN Neutron params if enabled
    if vxlan is True:
        env_config['parameter_defaults'].update({
            'NeutronEnableTunnelling': True,
            'NeutronTunnelTypes': 'vxlan',
            'NeutronBridgeMappings': 'datacentre:br-ex',
            'NeutronNetworkType': 'vxlan',
            'NeutronTypeDrivers': 'vxlan,flat'
        })
        del env_config['parameter_defaults']['NeutronNetworkVLANRanges']
    elif bond['bond_enable'] is False:
        env_config['parameter_defaults'].update({
            'NeutronBridgeMappings':
            'datacentre:br-ex,{0}:br-{0}'.format(
                 neutron['physnet_vlan']),
            'NeutronNetworkVLANRanges': '{0}:{1}'.format(
                                        neutron['physnet_vlan'],
                                        neutron['physnet_vlan_range'])
        })
    else:
        env_config['parameter_defaults'].update({
            'NeutronNetworkVLANRanges': '{0}:{1}'.format(
                                        neutron['physnet_vlan'],
                                        neutron['physnet_vlan_range']),
            'NeutronBridgeMappings':
            'datacentre:br-ex,{0}:br-bond'.format(neutron['physnet_vlan'])
        })

    # Generate environment configuration yaml
    with open(EnvParams.ENV_YAML_FILE_PATH, "w") as f:
        yaml.dump(
            env_config,
            f,
            default_flow_style=False,
            width=4,
            indent=4)

    output_msg = "Generated {0} configurations file".format(
        EnvParams.ENV_YAML_FILE) \
        + " to be used in overcloud deployment under {0}".format(
        EnvParams.ENV_CONF_DIR_PATH)
    logging.info(output_msg)
    print output_msg

if __name__ == '__main__':
    check_rules()
    generate_yaml()
