#!/usr/bin/env python
# Copyright 2016 Mellanox Technologies, Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import yaml
import os
import time
import sys
import argparse
import textwrap
import logging
from params import EnvParams
from utils import run_command
from utils import load_file
from utils import check_bool

logging.basicConfig(format='%(asctime)s %(filename)s %(message)s',
                    level=logging.DEBUG, filename=EnvParams.LOG_FILE)

parser = argparse.ArgumentParser(
    prog=os.path.basename(sys.argv[0]),
    formatter_class=argparse.RawDescriptionHelpFormatter,
    description=textwrap.dedent('''\
             Available Modes to choose from:
             -------------------------------
                - vlan
                - vxlan

            Set the required environment configurations in {0}
            Set the required network configurations in {1}
            '''.format(EnvParams.CONF_FILE_NAME,
                       EnvParams.NETWORK_CONF_FILE_NAME)))
args = parser.parse_args()

# Load configurations
params = load_file(EnvParams.CONF_FILE_PATH)
bond_enabled = check_bool(params['Bond']['bond_enable'])
network_type = params['Neutron']['network_type'].lower()
neutron_physnet_vlan = params['Neutron']['physnet_vlan']


def vxlan_tenant_section():

    vxlan_d = {
        "use_dhcp": False,
        "type": "interface",
        "name": {"get_param": "TenantInterface"},
        "addresses": [
             {
                "ip_netmask": {"get_param": "TenantIpSubnet"}
             }
                     ]
    }

    return vxlan_d


def bonding():

    bonding_d = {
       "type": "ovs_bridge",
       "name": "br-bond",
       "dns_servers": {"get_param": "DnsServers"},
       "addresses": [
            {
                 "ip_netmask": {"get_param": "TenantIpSubnet"}
            }
                    ],

       "members": [
            {
                "type": "ovs_bond",
                "name": "bond1",
                "ovs_options": {"get_param": "BondInterfaceOvsOptions"},
                "members": [
                    {
                        "type": "interface",
                        "name": {"get_param": "StorageInterface"}
                    },
                    {
                        "type": "interface",
                        "name": {"get_param": "TenantInterface"},
                        "primary": True
                    }
                           ],
            },
            {
                "type": "vlan",
                "device": "bond1",
                "vlan_id": {"get_param": "StorageNetworkVlanID"},
                "addresses": [
                    {
                        "ip_netmask": {"get_param": "StorageIpSubnet"}
                    }
                            ]
            },
            {
                "type": "vlan",
                "device": "bond1",
                "vlan_id": {"get_param": "TenantNetworkVlanID"}
            }
                   ]
                 }

    return bonding_d


def replace_to_vxlan(role_config):

    # Find the Tenant Interface section and replace with vlan
    resources = role_config['resources']
    node_config = resources['OsNetConfigImpl']['properties']['config']
    role_network_config = node_config['os_net_config']['network_config']
    for conf in role_network_config:
        if 'members' in conf:
            for item in conf['members']:
                if item['name']['get_param'] == 'TenantInterface':
                    role_network_config.remove(conf)
                    role_network_config.append(vxlan_tenant_section())
                    break


def add_bonding(role_config):

    resources = role_config['resources']
    node_config = resources['OsNetConfigImpl']['properties']['config']
    role_network_config = node_config['os_net_config']['network_config']
    for conf in role_network_config:
        if 'get_param' in conf['name']:
            if conf['name']['get_param'] == "StorageInterface":
                role_network_config.remove(conf)
    for conf in role_network_config:
        if 'members' in conf:
            for item in conf['members']:
                if item['name']['get_param'] == 'TenantInterface':
                    role_network_config.remove(conf)
    role_network_config.append(bonding())


def replace_bridge_name(role_config):

    # Find the Tenant Interface section and replace with vlan
    resources = role_config['resources']
    node_config = resources['OsNetConfigImpl']['properties']['config']
    role_network_config = node_config['os_net_config']['network_config']
    for conf in role_network_config:
        if 'members' in conf:
            for item in conf['members']:
                if item['name']['get_param'] == 'TenantInterface':
                    conf['name'] = 'br-{0}'.format(neutron_physnet_vlan)
                    break


def remove_tenant(role_config):

    resources = role_config['resources']
    node_config = resources['OsNetConfigImpl']['properties']['config']
    role_network_config = node_config['os_net_config']['network_config']
    for conf in role_network_config:
        if conf['name'] == 'br-bond':
            del conf["addresses"]
            for item in conf['members']:
                if 'vlan_id' in item:
                    if item['vlan_id']['get_param'] == 'TenantNetworkVlanID':
                        conf['members'].remove(item)


def generate_network_yamls():

    controller_config = load_file(os.path.join(
        EnvParams.NETWORK_CONF_DIR_PATH, 'controller-template.yaml'))
    compute_config = load_file(os.path.join(
        EnvParams.NETWORK_CONF_DIR_PATH, 'compute-template.yaml'))
    cinder_storage_config = load_file(os.path.join(
        EnvParams.NETWORK_CONF_DIR_PATH, 'cinder-storage-template.yaml'))

    # In case of the network type is vxlan
    if network_type == 'vxlan':
        replace_to_vxlan(controller_config)
        replace_to_vxlan(compute_config)

    # In case of the bond is enabled
    if bond_enabled is True:
        add_bonding(controller_config)
        add_bonding(compute_config)
        add_bonding(cinder_storage_config)
        remove_tenant(cinder_storage_config)

    # In case of network type is vlan and bond is disabled
    if network_type == 'vlan' and bond_enabled is False:
        replace_bridge_name(controller_config)
        replace_bridge_name(compute_config)

    edited_files_configs = [(os.path.join(EnvParams.NETWORK_CONF_DIR_PATH,
                                          "compute.yaml"), compute_config),
                            (os.path.join(
                             EnvParams.NETWORK_CONF_DIR_PATH,
                             "controller.yaml"), controller_config),
                            (os.path.join(
                             EnvParams.NETWORK_CONF_DIR_PATH,
                             "cinder-storage.yaml"), cinder_storage_config)]
    files = list()

    # Generate environment configuration yaml
    for yaml_file, config in edited_files_configs:
        with open(yaml_file, "w") as f:
            yaml.dump(
                config,
                f,
                default_flow_style=False,
                width=2,
                indent=2)
        files.append(yaml_file)

    output_msg = "Generated YAML files {0} to be used in {1}.".format(
                 str(files).translate(None, "'[]"),
                 EnvParams.NETWORK_CONF_FILE_PATH)

    logging.info(output_msg)
    print output_msg

if __name__ == '__main__':
    generate_network_yamls()
