#!/bin/bash -x
# Copyright 2016 Mellanox Technologies, Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.


readonly SCRIPT_DIR=$(dirname "$0")
source /opt/mlnx_scripts/common
readonly KERNEL_VERSION="$(uname -r)"
readonly OFED_BASE_DIR="/opt/ofed"
readonly OFED_SRC_DIR="${OFED_BASE_DIR}"
readonly OFED_SUCCESS_FILE="${OFED_BASE_DIR}/.success"
readonly OFED_INFO="/usr/bin/ofed_info"
OFED_DIR=$OFED_SRC_DIR

function is_ofed_installed () {
  if [ -f ${OFED_SUCCESS_FILE} ] && [ -x ${OFED_INFO} ] && ( ${OFED_INFO} > /dev/null 2>&1 ); then
    installed_ofed_version=`${OFED_INFO} -s`
    logger_print info "OFED is already installed: ${installed_ofed_version}"
    return 0
  else
    return 1
  fi
}

function install_mlnx_ofed_src () {
  logger_print info "Installing ${OFED_PACKAGE_NAME} source code"
  if [ "$DISTRO" == "redhat" ]; then
    yum install ${OFED_PACKAGE_NAME} -y
  elif [ "$DISTRO" == "ubuntu" ]; then
    apt-get install ${OFED_PACKAGE_NAME} -y
  fi
  if [ $? -ne 0 ]; then
    logger_print error "Failed installing ${OFED_PACKAGE_NAME} package"
    exit 1
  fi
}

function add_kernel_support () {
  OFED_ADD_KERNEL_SUPPORT_SCRIPT="${OFED_DIR}/mlnx_add_kernel_support.sh"
  if [ ! -x $OFED_ADD_KERNEL_SUPPORT_SCRIPT ]; then
    logger_print error "Failed to find $OFED_ADD_KERNEL_SUPPORT_SCRIPT"
    exit 1
  fi
  OFED_VERSION=$(cat ${OFED_DIR}/.mlnx)
  OFED_ARCH=$(cat ${OFED_DIR}/.arch)
  OFED_DISTRO=$(cat ${OFED_DIR}/distro)
  RECOMPILED_OFED_NAME="MLNX_OFED_LINUX-${OFED_VERSION}-${OFED_DISTRO}-${OFED_ARCH}-ext"
  RECOMPILED_OFED_DIR="${OFED_BASE_DIR}/${RECOMPILED_OFED_NAME}"

  # Recompile OFED in case original OFED at $OFED_DIR doesn't support the existing kernel
  if ( ! grep -Fxq ${KERNEL_VERSION}  ${OFED_DIR}/.supported_kernels ); then
    if [ ! -d ${RECOMPILED_OFED_DIR} ] || \
       ( ! grep -Fxq ${KERNEL_VERSION} ${RECOMPILED_OFED_DIR}/.supported_kernels); then
      logger_print info "Recompiling OFED for kernel ${KERNEL_VERSION}"
      ${OFED_DIR}/mlnx_add_kernel_support.sh --force --yes --make-tgz --mlnx_ofed ${OFED_DIR}
      recompiled_ofed_archive=/tmp/${RECOMPILED_OFED_NAME}.tgz
      tar zxf $recompiled_ofed_archive -C ${OFED_BASE_DIR}
      rm -f $recompiled_ofed_archive
    fi
    OFED_DIR=$RECOMPILED_OFED_DIR
  fi
}

function install_ofed () {
  OFED_INSTALL_SCRIPT="${OFED_DIR}/mlnxofedinstall"
  if [ ! -f $OFED_INSTALL_SCRIPT ]; then
    logger_print error "Failed to find $OFED_INSTALL_SCRIPT"
    exit 1
  fi

  logger_print info "Installing OFED drivers"
  OFED_INSTALL_SCRIPT_CMD="/usr/bin/perl ${OFED_INSTALL_SCRIPT}"
  ${OFED_INSTALL_SCRIPT_CMD} --force

  rc=$?
  if [ $rc -ne 0 ]; then
    logger_print error "Failed execute ${OFED_INSTALL_SCRIPT_CMD} error code ${rc}"
    exit 1
  else
    touch ${OFED_SUCCESS_FILE}
  fi
}

function install_prequisits (){
  pckges="autoconf automake chrpath gcc cpp swig numactl graphviz \
          m4 make mc wget boost boost-devel boost-regex libxml2-python \
          redhat-rpm-config rpm-build tcl gcc-gfortran tcsh tk sqlite \
          bind-utils createrepo"
  for package in $pckges; do
    yum install -y $package
  done
}
function extract_ofed (){
  mkdir /mnt/ofed
  mount -o loop /opt/MLNX* /mnt/ofed
  cp -a /mnt/ofed/. /opt/ofed/
  umount /mnt/ofed
  rm -rf /mnt/ofed

}

if ! is_ofed_installed; then

  # Install prequisit packages
  install_prequisits

  # Extract the ISO file under /opt/ofed
  extract_ofed

  # Add support for the current kernel
  add_kernel_support

  # First install OFED without SR-IOV
  install_ofed

fi

shutdown -r && exit 0
