#!/usr/bin/env python
# Copyright 2016 Mellanox Technologies, Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys
import logging
import argparse
import textwrap
from collections import OrderedDict
from utils import run_command
from utils import load_file
from utils import check_bool
from utils import install_libguestsfs
from utils import check_libvirtd
from params import EnvParams
from params import OvercloudParams
from params import PackageParams


parser = argparse.ArgumentParser(
    prog=os.path.basename(sys.argv[0]),
    formatter_class=argparse.RawDescriptionHelpFormatter,
    description=textwrap.dedent('''\
             Log file: {0}

             '''.format(EnvParams.LOG_FILE_PATH)))

parser.add_argument(
    "--iso_ofed_path",
    help='path to the OFED iso file',
    type=str,
    required=True)
parser.add_argument(
    "--overcloud_images_path",
    help='path to the overcloud images',
    type=str,
    required=True)
parser.add_argument(
    "--root_password",
    help='set root password for the overcloud nodes',
    type=str,
    required=True)
parser.add_argument(
    "--lldp_package_path",
    help='Optional path to install specific lldpd package',
    type=str,
    required=False)
parser.add_argument(
    "--python_mlnx_package_path",
    help='Optional path to install specific python-networking-mlnx package',
    type=str,
    required=False)
parser.add_argument(
    "--interfaces_drivers",
    help=argparse.SUPPRESS,
    type=str,
    required=False)

args = parser.parse_args()
ovc_image = os.path.join(args.overcloud_images_path,
                         EnvParams.OVC_FULL_IMAGE_NAME)


def load_overcloud_conf():
    """
    Load overcloud configuartions.
    """
    config = load_file(EnvParams.CONF_FILE_PATH)
    return config


def upload_OFED():
    """
    Upload OFED iso to the overcloud-full image.
    """
    ofed_d = extract_ofed_info()
    if 'os_version' in ofed_d:
        check_ovc_target_OS(ofed_d['os_version'].strip('rhel'))

    run_command('virt-customize -a {0} --mkdir {1}'
                .format(ovc_image,
                        OvercloudParams.OVC_OFED_DIR_PATH))

    run_command('virt-customize -a {0} --upload \
                {1}:{2}'.format(ovc_image,
                                args.iso_ofed_path,
                                OvercloudParams.OVC_MLNX_PATH))
    rename_ofed_iso(ofed_d)


def extract_ofed_info():
    """
    Extract ofed version, distro and arch from ofed files.
    """
    ofed_d = OrderedDict()
    run_command('sudo mkdir -p {0}'.format(EnvParams.MNT_OFED_DIR_PATH))
    run_command('sudo mount -o loop {0} {1}'
                .format(args.iso_ofed_path, EnvParams.MNT_OFED_DIR_PATH))

    if os.path.isfile(EnvParams.OFED_VERSION_PATH):
        ofed_d['ofed_version'] = open(EnvParams.OFED_VERSION_PATH) \
            .read().strip()
    if os.path.isfile(EnvParams.OFED_DISTRO_PATH):
        ofed_d['os_version'] = open(EnvParams.OFED_DISTRO_PATH).read().strip()
    if os.path.isfile(EnvParams.OFED_ARCH_PATH):
        ofed_d['arch'] = open(EnvParams.OFED_ARCH_PATH).read().strip()

    run_command('sudo umount {0}'.format(EnvParams.MNT_OFED_DIR_PATH))
    run_command('sudo rm -rf {0}'.format(EnvParams.MNT_OFED_DIR_PATH))
    return ofed_d


def check_ovc_target_OS(ofed_distro):
    """
    Check OFED OS version vs. overcloud target OS version.
    """
    run_command(
        "virt-customize -a {0} --run-command 'lsb_release -rs > {1}'"
        .format(ovc_image,
                os.path.join(OvercloudParams.OVC_TARGET_OS_DIR_PATH,
                             OvercloudParams.OVC_TARGET_OS)))

    run_command('virt-copy-out -a {0} {1} {2}'
                .format(ovc_image,
                        os.path.join(OvercloudParams.OVC_TARGET_OS_DIR_PATH,
                                     OvercloudParams.OVC_TARGET_OS),
                        EnvParams.TMP_FILES_DIR_PATH))

    if ofed_distro != open(os.path.join(EnvParams.TMP_FILES_DIR_PATH,
                                        OvercloudParams.OVC_TARGET_OS)) \
            .read().strip():
        run_command("virt-customize -a {0} --run-command \
                   'cat /etc/redhat-release > {1}' "
                    .format(ovc_image,
                            OvercloudParams.OVC_TARGET_OS))

        run_command('virt-copy-out -a {0} {1} {2}'
                    .format(ovc_image,
                            os.path.join(
                                OvercloudParams.OVC_TARGET_OS_DIR_PATH,
                                OvercloudParams.OVC_TARGET_OS),
                            EnvParams.TMP_FILES_DIR_PATH))
        target_os = open(os.path.join(EnvParams.TMP_FILES_DIR_PATH,
                                      OvercloudParams.OVC_TARGET_OS)).read()
        if ofed_distro not in target_os:
            error_msg = 'Please use compatible OFED version for {0}\n' \
                        .format(target_os)
            logging.error(error_msg)
            sys.stderr.write(error_msg)
            sys.exit(1)


def rename_ofed_iso(ofed_d):
    """
    Rename ofed iso on overcloud if needed.
    """
    ofed_file = os.path.basename(os.path.normpath(args.iso_ofed_path))

    ofed_new_name = OvercloudParams.OFED_PREFIX
    for key, value in ofed_d.items():
        ofed_new_name = '{0}-{1}'.format(ofed_new_name, value)
    ofed_new_name = '{0}.iso'.format(ofed_new_name)

    if ofed_file != ofed_new_name:
        run_command("virt-customize -a {0} --run-command \
                   'mv {1} {2}' "
                    .format(ovc_image,
                            os.path.join(OvercloudParams.OVC_MLNX_PATH,
                                         ofed_file),
                            os.path.join(OvercloudParams.OVC_MLNX_PATH,
                                         ofed_new_name)))


def set_root_password():
    """
    Set root password.
    """
    run_command('virt-customize -a {0} --root-password password:{1}'
                .format(ovc_image,
                        args.root_password))


def upload_ovc_scripts():
    """
    Upload common and restart_service files to overcloud-full image.
    """
    run_command('virt-customize -a {0} --mkdir {1}'
                .format(ovc_image,
                        OvercloudParams.OVC_MLNX_SCRIPTS_DIR_PATH))

    run_command(
        'export LIBGUESTFS_BACKEND=direct && virt-customize -a {0} \
         --upload {1}:{2}'
        .format(ovc_image,
                EnvParams.COMMON_SCRIPT_PATH,
                OvercloudParams.OVC_MLNX_SCRIPTS_DIR_PATH))
    run_command(
        'virt-customize -a {0} \
         --upload {1}:{2}'
        .format(ovc_image,
                EnvParams.RESTART_SERVICE_PATH,
                OvercloudParams.OVC_MLNX_SCRIPTS_DIR_PATH))


def install_NEO_pkgs(sriov_interface):
    """
    if NEO is enabled, install lldpd and python-networking-mlnx to
    overcloud-full image.
    """

    # Add MLNX Repo.
    add_future_yum(PackageParams.MLNX_REPO_URL,
                   PackageParams.MLNX_REPO)

    # Create Firstboot Script.
    f = open(EnvParams.FIRSTBOOT_SCRIPT_PATH, 'w')

    f.write(firstboot_install(PackageParams.MLNX_REPO, PackageParams.LIBZMQ))
    f.write(firstboot_install(PackageParams.MLNX_REPO, PackageParams.PYZMQ))

    # Add lldpd interface configurations
    f.write("echo 'configure system interface pattern {0}' >> {1}\n"
            .format(sriov_interface,
                    OvercloudParams.LLDPD_CONF_FILE_PATH))

    # Create directory for packages provided by user.
    if args.lldp_package_path or args.python_mlnx_package_path:
        run_command('virt-customize -a {0} --mkdir {1}'
                    .format(ovc_image,
                            OvercloudParams.OVC_MLNX_PKGS_DIR_PATH))
        f.write('yum install -y {0}/*\n'
                .format(OvercloudParams.OVC_MLNX_PKGS_DIR_PATH))

    # Upload lldpd provided by user or install from repo.
    if args.lldp_package_path:
        run_command(
            'virt-customize -a {0} --copy-in {1}:{2}'
            .format(ovc_image,
                    args.lldp_package_path,
                    OvercloudParams.OVC_MLNX_PKGS_DIR_PATH))
    else:
        f.write(firstboot_install(PackageParams.MLNX_REPO,
                                  PackageParams.LLDPD))

    # Upload python-networking-mlnx provided by user or install from repo.
    if args.python_mlnx_package_path:
        run_command(
            'virt-customize -a {0} --copy-in {1}:{2}'
            .format(ovc_image,
                    args.python_mlnx_package_path,
                    OvercloudParams.OVC_MLNX_PKGS_DIR_PATH))

    else:
        add_future_yum(PackageParams.PYTHON_NET_MLNX_URL,
                       PackageParams.PYTHON_NETWORKING_MLNX_REPO)
        f.write(firstboot_install(PackageParams.PYTHON_NETWORKING_MLNX_REPO,
                                  PackageParams.PYTHON_NET_MLNX))

    # Add Firstboot Script.
    f.close()
    run_command("virt-customize -a {0} --firstboot {1}"
                .format(ovc_image,
                        EnvParams.FIRSTBOOT_SCRIPT_PATH))


def add_future_yum(repo_url, repo_name):
    """
    Add repo and disable repo.
    """
    # Add repository to overcloud-full image.
    run_command("virt-customize -a {0} --run-command \
                'yum-config-manager --add-repo {1}'"
                .format(ovc_image,
                        repo_url))

    # Disable the added repository.
    run_command("virt-customize -a {0} --run-command \
                'yum-config-manager --disable {1}'"
                .format(ovc_image,
                        repo_name))


def firstboot_install(repo_name, package):
    """
    Generate firstboot command to install packages from specific repo.
    """

    # Install package from the added repo.
    cmd = "yum install -y --nogpgcheck {0} --enablerepo={1}\n" \
          .format(package,
                  repo_name)

    return cmd


def update_overcloud_images():
    """
    Upload and update existing overcloud images.
    """
    run_command('source {0} && cd {1} && openstack overcloud image upload --image-path {1} \
                --update-existing'.format(os.path.join('~', 'stackrc'),
                                          args.overcloud_images_path))


def prepare_ovc_image():

    logging.basicConfig(format='%(asctime)s  %(filename)s %(message)s',
                        level=logging.DEBUG, filename=EnvParams.LOG_FILE_PATH)

    run_command('mkdir -p {0}'.format(EnvParams.TMP_FILES_DIR_PATH))
    config = load_overcloud_conf()
    install_libguestsfs()
    check_libvirtd()
    upload_OFED()
    set_root_password()
    upload_ovc_scripts()

    config['Neo']['neo_enable'] = check_bool(config['Neo']['neo_enable'])
    if config['Neo']['neo_enable'] is True:
        install_NEO_pkgs(config['Sriov']['sriov_interface'])

    update_overcloud_images()

    output = 'Overcloud images prepared successfully, run introspection again'
    logging.info(output)
    sys.stderr.write(output + '\n')
    sys.exit(0)


if __name__ == '__main__':
    prepare_ovc_image()
