#!/usr/bin/env python
# Copyright 2016 Mellanox Technologies, Ltd
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import json
import yaml
import os
import subprocess
import sys
import argparse
import textwrap
import logging
from params import EnvParams

parser = argparse.ArgumentParser(
    prog=os.path.basename(sys.argv[0]),
    formatter_class=argparse.RawDescriptionHelpFormatter,
    description=textwrap.dedent('''\
             log file: {0}

             '''.format(EnvParams.LOG_FILE_PATH)))

parser.add_argument(
    "--path_to_swift_data",
    help='Path to swift-data directory',
    type=str,
    required=True)

args = parser.parse_args()


def run_command(cmd):

    try:
        p_output = subprocess.check_output(
            cmd, shell=True, stderr=subprocess.STDOUT)
        logging.info(p_output)
        return p_output

    except subprocess.CalledProcessError as e:
        print e.output
        logging.error(e.output)
        sys.exit(1)


def main():

    logging.basicConfig(
        format='%(asctime)s %(filename)s %(message)s',
        level=logging.DEBUG,
        filename=EnvParams.LOG_FILE_PATH,
        stream=sys.stdout)

    # save ironic nodes uuids in dict of uuid as key empty string value
    nodes_json = run_command('source {0} && ironic --json node-list'.format(
        EnvParams.STACKRC_FILE_PATH))
    roles_dict = {}
    for node in json.loads(nodes_json):
        if node['provision_state'].lower() == 'available':
            roles_dict[node['uuid']] = ''

    # update roles_dict with nodes' role as value for each key
    profiles_json = run_command(
        'source {0} && openstack overcloud profiles list -f json'.format(
            EnvParams.STACKRC_FILE_PATH))
    for profile_dict in json.loads(profiles_json):
        node_hash = profile_dict['Node UUID']
        profile = profile_dict['Current Profile']
        if node_hash in roles_dict:
            roles_dict[node_hash] = profile

    interfaces_dict = {}

    # load profile, interfaces and drivers for each ironic node into a yaml
    # file
    for node_hash in roles_dict:

        profile_index = node_hash
        profile_name = roles_dict[node_hash]
        if profile_name != 'None':
            profile_index = '%s/%s' % (profile_name, node_hash)

        interfaces_dict[profile_index] = {}
        interfaces_dict[profile_index]['interfaces'] = {}

        # list ironic node interfaces and drivers
        try:
            with open(args.path_to_swift_data +
                      "/inspector_data-" + node_hash, 'r') as stream:
                inspector_data = yaml.load(stream)

                for interface in inspector_data['extra']['network']:
                    if interface not in inspector_data['all_interfaces']:
                        logging.info('Interface {0} is skipped'
                                     .format(interface))
                        continue
                    ifc = dict()
                    src_data = inspector_data['extra']['network'][interface]
                    ifc['driver'] = src_data['driver']
                    ifc['product'] = src_data['product']
                    ifc['mac'] = \
                        inspector_data['all_interfaces'][interface]['mac']
                    if 'speed' in src_data:
                        ifc['speed'] = src_data['speed']
                    else:
                        ifc['speed'] = 'N/A'
                    interfaces_dict[profile_index][
                        'interfaces'][interface] = ifc

        except IOError as e:
            print args.path_to_swift_data
            logging.error("I/O error({0}): {1}".format(e.errno, e.strerror))
            print "I/O error({0}): {1}".format(e.errno, e.strerror)
            print 'Please update ' + args.path_to_swift_data + \
                  ' with inspector_data files of the ironic ' + \
                  'nodes\nLog file: ', EnvParams.LOG_FILE_PATH
            exit(1)

    with open(EnvParams.INTERFACES_DRIVERS_YAML_FILE_PATH, "w") as f:
        yaml.safe_dump(
            interfaces_dict,
            f,
            default_flow_style=False,
            width=8,
            indent=8)
    output_msg = 'Check {0} for the summary of '.format(
                 EnvParams.INTERFACES_DRIVERS_YAML_FILE_PATH) + \
        'available nodes interfaces.'
    print output_msg
    logging.info(output_msg)


if __name__ == '__main__':
    main()
